"""Модуль, который предоставляет мапперы для унифицированных клиентов и вебсокет-менеджеров."""

__all__ = [
    "get_uni_client",
    "get_uni_websocket_manager",
    "get_exchange_info",
]


from ._abc import IExchangeInfo, IUniClient, IUniWebsocketManager
from .binance import ExchangeInfo as BinanceExchangeInfo
from .binance import UniClient as BinanceUniClient
from .binance import UniWebsocketManager as BinanceUniWebsocketManager
from .bitget import ExchangeInfo as BitgetExchangeInfo
from .bitget import UniClient as BitgetUniClient
from .bitget import UniWebsocketManager as BitgetUniWebsocketManager
from .bybit import ExchangeInfo as BybitExchangeInfo
from .bybit import UniClient as BybitUniClient
from .bybit import UniWebsocketManager as BybitUniWebsocketManager
from .enums import Exchange
from .exceptions import NotSupported
from .gate import ExchangeInfo as GateioExchangeInfo
from .gate import UniClient as GateioUniClient
from .gate import UniWebsocketManager as GateioUniWebsocketManager
from .hyperliquid import ExchangeInfo as HyperliquidExchangeInfo
from .hyperliquid import UniClient as HyperliquidUniClient
from .hyperliquid import UniWebsocketManager as HyperliquidUniWebsocketManager
from .mexc import ExchangeInfo as MexcExchangeInfo
from .mexc import UniClient as MexcUniClient
from .mexc import UniWebsocketManager as MexcUniWebsocketManager
from .okx import ExchangeInfo as OkxExchangeInfo
from .okx import UniClient as OkxUniClient
from .okx import UniWebsocketManager as OkxUniWebsocketManager

_UNI_CLIENT_MAPPER: dict[Exchange, type[IUniClient]] = {
    Exchange.BINANCE: BinanceUniClient,
    Exchange.BITGET: BitgetUniClient,
    Exchange.BYBIT: BybitUniClient,
    Exchange.GATE: GateioUniClient,
    Exchange.HYPERLIQUID: HyperliquidUniClient,
    Exchange.MEXC: MexcUniClient,
    Exchange.OKX: OkxUniClient,
}
"""Маппер, который связывает биржу и реализацию унифицированного клиента."""

_UNI_WS_MANAGER_MAPPER: dict[Exchange, type[IUniWebsocketManager]] = {
    Exchange.BINANCE: BinanceUniWebsocketManager,
    Exchange.BITGET: BitgetUniWebsocketManager,
    Exchange.BYBIT: BybitUniWebsocketManager,
    Exchange.GATE: GateioUniWebsocketManager,
    Exchange.HYPERLIQUID: HyperliquidUniWebsocketManager,
    Exchange.MEXC: MexcUniWebsocketManager,
    Exchange.OKX: OkxUniWebsocketManager,
}
"""Маппер, который связывает биржу и реализацию унифицированного вебсокет-менеджера."""

_EXCHANGE_INFO_MAPPER: dict[Exchange, type[IExchangeInfo]] = {
    Exchange.BINANCE: BinanceExchangeInfo,
    Exchange.BITGET: BitgetExchangeInfo,
    Exchange.BYBIT: BybitExchangeInfo,
    Exchange.GATE: GateioExchangeInfo,
    Exchange.HYPERLIQUID: HyperliquidExchangeInfo,
    Exchange.MEXC: MexcExchangeInfo,
    Exchange.OKX: OkxExchangeInfo,
}
"""Маппер, который связывает биржу и реализацию сборщика информации о тикерах на бирже."""


def get_uni_client(exchange: Exchange) -> type[IUniClient]:
    """Возвращает унифицированный клиент для указанной биржи.

    Параметры:
        exchange (`Exchange`): Биржа.

    Возвращает:
        `type[IUniClient]`: Унифицированный клиент для указанной биржи.
    """
    try:
        return _UNI_CLIENT_MAPPER[exchange]
    except KeyError as e:
        raise NotSupported(f"Unsupported exchange: {exchange}") from e


def get_uni_websocket_manager(exchange: Exchange) -> type[IUniWebsocketManager]:
    """Возвращает унифицированный вебсокет-менеджер для указанной биржи.

    Параметры:
        exchange (`Exchange`): Биржа.

    Возвращает:
        `type[IUniWebsocketManager]`: Унифицированный вебсокет-менеджер для указанной биржи.
    """
    try:
        return _UNI_WS_MANAGER_MAPPER[exchange]
    except KeyError as e:
        raise NotSupported(f"Unsupported exchange: {exchange}") from e


def get_exchange_info(exchange: Exchange) -> type[IExchangeInfo]:
    """Возвращает унифицированный интерфейс для получения информации о бирже.

    Параметры:
        exchange (`Exchange`): Биржа.

    Возвращает:
        `type[IExchangeInfo]`: Унифицированный интерфейс для получения информации о бирже.
    """
    try:
        return _EXCHANGE_INFO_MAPPER[exchange]
    except KeyError as e:
        raise NotSupported(f"Unsupported exchange: {exchange}") from e



__all__ = ["Client"]

import json
import time
import warnings
from typing import Any

from unicex._base import BaseClient
from unicex.exceptions import NotAuthorized
from unicex.types import RequestMethod
from unicex.utils import dict_to_query_string, filter_params, generate_hmac_sha256_signature


class Client(BaseClient):
    """Клиент для работы с Binance API."""

    _BASE_SPOT_URL: str = "https://api.binance.com"
    """Базовый URL для REST API Binance Spot."""

    _BASE_FUTURES_URL: str = "https://fapi.binance.com"
    """Базовый URL для REST API Binance Futures."""

    _RECV_WINDOW: int = 5000
    """Стандартный интервал времени для получения ответа от сервера."""

    def _get_headers(self, method: RequestMethod) -> dict:
        """Возвращает заголовки для запросов к Binance API."""
        headers = {"Accept": "application/json"}
        if self._api_key:  # type: ignore[attr-defined]
            headers["X-MBX-APIKEY"] = self._api_key  # type: ignore[attr-defined]
        if method in ["POST", "PUT", "DELETE"]:
            headers.update({"Content-Type": "application/x-www-form-urlencoded"})
        return headers

    def _prepare_payload(
        self,
        *,
        method: RequestMethod,
        signed: bool,
        params: dict[str, Any] | None,
    ) -> tuple[dict[str, Any], dict[str, Any] | None]:
        """Подготавливает payload и заголовки для запроса.

        Если signed=True:
            - добавляет подпись и все обязательные параметры в заголовки

        Если signed=False:
            - возвращает только отфильтрованные params.

        Параметры:
            method (`RequestMethod`): Метод запроса.
            signed (`bool`): Нужно ли подписывать запрос.
            params (`dict | None`): Параметры для query string.

        Возвращает:
            tuple:
                - payload (`dict`): Параметры/тело запроса с подписью (если нужно).
                - headers (`dict | None`): Заголовки для запроса или None.
        """
        # Фильтруем параметры от None значений
        params = filter_params(params) if params else {}

        # Получаем заголовки для запроса
        headers = self._get_headers(method)

        if not signed:
            return {"params": params}, headers

        if not self.is_authorized():
            raise NotAuthorized("Api key and api secret is required to private endpoints")

        # Объединяем все параметры в payload
        payload = {**params}
        payload["timestamp"] = int(time.time() * 1000)
        payload["recvWindow"] = self._RECV_WINDOW

        # Генерируем подпись
        query_string = dict_to_query_string(payload)
        payload["signature"] = generate_hmac_sha256_signature(
            self._api_secret,  # type: ignore[attr-defined]
            query_string,
            "hex",
        )

        return payload, headers

    async def _make_request(
        self,
        method: RequestMethod,
        url: str,
        signed: bool = False,
        *,
        params: dict[str, Any] | None = None,
    ) -> Any:
        """Выполняет HTTP-запрос к эндпоинтам Binance API.

        Если signed=True, формируется подпись для приватных endpoint'ов:
            - Если метод запроса "GET" — подпись добавляется в параметры запроса.
            - Если метод запроса "POST" | "PUT" | "DELETE" — подпись добавляется в тело запроса.

        Если signed=False, запрос отправляется как публичный.

        Параметры:
            method (`str`): HTTP метод ("GET", "POST", "DELETE" и т.д.).
            url (`str`): Полный URL эндпоинта Binance API.
            signed (`bool`): Нужно ли подписывать запрос.
            params (`dict | None`): Query-параметры.

        Возвращает:
            `dict`: Ответ в формате JSON.
        """
        payload, headers = self._prepare_payload(method=method, signed=signed, params=params)

        if not signed:
            return await super()._make_request(method=method, url=url, **payload)

        return await super()._make_request(method=method, url=url, params=payload, headers=headers)

    async def request(
        self, method: RequestMethod, url: str, params: dict, data: dict, signed: bool
    ) -> dict:
        """Специальный метод для выполнения запросов на эндпоинты, которые не обернуты в клиенте.

        Параметры:
            method (`str`): HTTP метод ("GET", "POST", "DELETE" и т.д.).
            url (`str`): Полный URL эндпоинта Binance API.
            signed (`bool`): Нужно ли подписывать запрос.
            params (`dict | None`): Query-параметры.
            data (`dict | None`): Тело запроса.

        Возвращает:
            `dict`: Ответ в формате JSON.
        """
        return await self._make_request(method=method, url=url, params=params, signed=signed)

    # topic: general endpoints

    async def ping(self) -> dict:
        """Проверка подключения к REST API.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/general-endpoints#test-connectivity
        """
        url = self._BASE_SPOT_URL + "/api/v3/ping"

        return await self._make_request("GET", url)

    async def server_time(self) -> dict:
        """Получение серверного времени.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/general-endpoints#check-server-time
        """
        url = self._BASE_SPOT_URL + "/api/v3/time"

        return await self._make_request("GET", url)

    async def exchange_info(self) -> dict:
        """Получение информации о символах рынка и текущих правилах биржевой торговли.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/general-endpoints#exchange-information
        """
        url = self._BASE_SPOT_URL + "/api/v3/exchangeInfo"

        return await self._make_request("GET", url)

    # topic: market data endpoints

    async def depth(self, symbol: str, limit: int | None = None) -> dict:
        """Получение книги ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#order-book
        """
        url = self._BASE_SPOT_URL + "/api/v3/depth"
        params = {"symbol": symbol, "limit": limit}

        return await self._make_request("GET", url, params=params)

    async def trades(self, symbol: str, limit: int | None = None) -> list[dict]:
        """Получение последних сделок.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#recent-trades-list
        """
        url = self._BASE_SPOT_URL + "/api/v3/trades"
        params = {"symbol": symbol, "limit": limit}

        return await self._make_request("GET", url, params=params)

    async def historical_trades(
        self, symbol: str, limit: int | None = None, from_id: int | None = None
    ) -> list[dict]:
        """Исторические сделки.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#old-trade-lookup
        """
        url = self._BASE_SPOT_URL + "/api/v3/historicalTrades"
        params = {"symbol": symbol, "limit": limit, "fromId": from_id}

        return await self._make_request("GET", url, params=params)

    async def agg_trades(
        self,
        symbol: str,
        from_id: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение агрегированных сделок.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#compressedaggregate-trades-list
        """
        url = self._BASE_SPOT_URL + "/api/v3/aggTrades"
        params = {
            "symbol": symbol,
            "fromId": from_id,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def klines(
        self,
        symbol: str,
        interval: str,
        start_time: int | None = None,
        end_time: int | None = None,
        time_zone: str | None = None,
        limit: int | None = None,
    ) -> list[list]:
        """Получение исторических свечей.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#klinecandlestick-data
        """
        url = self._BASE_SPOT_URL + "/api/v3/klines"
        params = {
            "symbol": symbol,
            "interval": interval,
            "startTime": start_time,
            "endTime": end_time,
            "timeZone": time_zone,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def ui_klines(
        self,
        symbol: str,
        interval: str,
        start_time: int | None = None,
        end_time: int | None = None,
        time_zone: str | None = None,
        limit: int | None = None,
    ) -> list[list]:
        """Получение UI свечей (оптимизированы для отображения).

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#uiklines
        """
        url = self._BASE_SPOT_URL + "/api/v3/uiKlines"
        params = {
            "symbol": symbol,
            "interval": interval,
            "startTime": start_time,
            "endTime": end_time,
            "timeZone": time_zone,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def avg_price(self, symbol: str) -> dict:
        """Получение текущей средней цены символа.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#current-average-price
        """
        url = self._BASE_SPOT_URL + "/api/v3/avgPrice"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, params=params)

    async def ticker_24hr(
        self,
        symbol: str | None = None,
        symbols: list[str] | None = None,
        type: str | None = None,
    ) -> dict | list[dict]:
        """Получение статистики изменения цен и объема за 24 часа.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#24hr-ticker-price-change-statistics
        """
        url = self._BASE_SPOT_URL + "/api/v3/ticker/24hr"
        params = {"symbol": symbol, "type": type, "symbols": symbols}

        return await self._make_request("GET", url, params=params)

    async def ticker_trading_day(
        self,
        symbol: str | None = None,
        symbols: list[str] | None = None,
        time_zone: str | None = None,
        type: str | None = None,
    ) -> dict | list[dict]:
        """Статистика изменения цен за торговый день.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#trading-day-ticker
        """
        url = self._BASE_SPOT_URL + "/api/v3/ticker/tradingDay"
        params = {
            "symbol": symbol,
            "symbols": symbols,
            "timeZone": time_zone,
            "type": type,
        }

        return await self._make_request("GET", url, params=params)

    async def ticker_price(
        self, symbol: str | None = None, symbols: list[str] | None = None
    ) -> dict | list[dict]:
        """Получение последней цены тикера(ов).

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#symbol-price-ticker
        """
        url = self._BASE_SPOT_URL + "/api/v3/ticker/price"
        params = {"symbol": symbol, "symbols": symbols}

        return await self._make_request("GET", url, params=params)

    async def ticker_book_ticker(
        self, symbol: str | None = None, symbols: list[str] | None = None
    ) -> dict | list[dict]:
        """Получение лучших цен bid/ask в книге ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#symbol-order-book-ticker
        """
        url = self._BASE_SPOT_URL + "/api/v3/ticker/bookTicker"
        params = {"symbol": symbol, "symbols": symbols}

        return await self._make_request("GET", url, params=params)

    async def ticker(
        self,
        symbol: str | None = None,
        symbols: list[str] | None = None,
        window_size: str | None = None,
        type: str | None = None,
    ) -> dict | list[dict]:
        """Статистика изменения цен в скользящем окне.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/market-data-endpoints#rolling-window-price-change-statistics
        """
        url = self._BASE_SPOT_URL + "/api/v3/ticker"
        params = {
            "symbol": symbol,
            "symbols": symbols,
            "windowSize": window_size,
            "type": type,
        }

        return await self._make_request("GET", url, params=params)

    # topic: trading endpoints

    async def order_create(
        self,
        symbol: str,
        side: str,
        type: str,
        quantity: float | None = None,
        quote_order_qty: float | None = None,
        price: float | None = None,
        stop_price: float | None = None,
        time_in_force: str | None = None,
        new_client_order_id: str | None = None,
        iceberg_qty: float | None = None,
        new_order_resp_type: str | None = None,
        self_trade_prevention_mode: str | None = None,
    ) -> dict:
        """Создание нового ордера на спот-рынке.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#new-order-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/order"
        params = {
            "symbol": symbol,
            "side": side,
            "type": type,
            "quantity": quantity,
            "quoteOrderQty": quote_order_qty,
            "price": price,
            "stopPrice": stop_price,
            "timeInForce": time_in_force,
            "newClientOrderId": new_client_order_id,
            "icebergQty": iceberg_qty,
            "newOrderRespType": new_order_resp_type,
            "selfTradePreventionMode": self_trade_prevention_mode,
        }

        # return await self._make_request("POST", url, True, params=params)
        return await self._make_request("POST", url, True, params=params)

    async def order_test(
        self,
        symbol: str,
        side: str,
        type: str,
        quantity: float | None = None,
        quote_order_qty: float | None = None,
        price: float | None = None,
        stop_price: float | None = None,
        time_in_force: str | None = None,
        new_client_order_id: str | None = None,
        iceberg_qty: float | None = None,
        new_order_resp_type: str | None = None,
        self_trade_prevention_mode: str | None = None,
    ) -> dict:
        """Тестирование нового ордера (не выполняется реально).

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#test-new-order-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/order/test"
        params = {
            "symbol": symbol,
            "side": side,
            "type": type,
            "quantity": quantity,
            "quoteOrderQty": quote_order_qty,
            "price": price,
            "stopPrice": stop_price,
            "timeInForce": time_in_force,
            "newClientOrderId": new_client_order_id,
            "icebergQty": iceberg_qty,
            "newOrderRespType": new_order_resp_type,
            "selfTradePreventionMode": self_trade_prevention_mode,
        }

        return await self._make_request("POST", url, True, params=params)

    async def order_cancel(
        self,
        symbol: str,
        order_id: int | None = None,
        orig_client_order_id: str | None = None,
        new_client_order_id: str | None = None,
    ) -> dict:
        """Отмена активного ордера на спот-рынке.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#cancel-order-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/order"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "origClientOrderId": orig_client_order_id,
            "newClientOrderId": new_client_order_id,
        }

        return await self._make_request("DELETE", url, True, params=params)

    async def orders_cancel_all(self, symbol: str) -> list[dict]:
        """Отмена всех активных ордеров по символу.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#cancel-all-open-orders-on-a-symbol-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/openOrders"
        params = {"symbol": symbol}

        return await self._make_request("DELETE", url, True, params=params)

    async def orders_open(self, symbol: str | None = None) -> list[dict]:
        """Получение всех активных ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#current-open-orders-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/openOrders"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    async def oco_order_create(
        self,
        symbol: str,
        side: str,
        quantity: str,
        list_client_order_id: str | None = None,
        # ABOVE ORDER
        above_type: str = "TAKE_PROFIT_LIMIT",
        above_client_order_id: str | None = None,
        above_price: str | None = None,
        above_stop_price: str | None = None,
        above_trailing_delta: int | None = None,
        above_time_in_force: str | None = None,
        above_iceberg_qty: str | None = None,
        above_strategy_id: int | None = None,
        above_strategy_type: int | None = None,
        # BELOW ORDER
        below_type: str = "STOP_LOSS_LIMIT",
        below_client_order_id: str | None = None,
        below_price: str | None = None,
        below_stop_price: str | None = None,
        below_trailing_delta: int | None = None,
        below_time_in_force: str | None = None,
        below_iceberg_qty: str | None = None,
        below_strategy_id: int | None = None,
        below_strategy_type: int | None = None,
        # EXTRA
        new_order_resp_type: str | None = None,
        self_trade_prevention_mode: str | None = None,
    ) -> dict:
        """Создание OCO ордера (новая версия).

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/trading-endpoints#new-order-list---oco-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/orderList/oco"

        params = {
            "symbol": symbol,
            "side": side,
            "quantity": quantity,
            "listClientOrderId": list_client_order_id,
            # ABOVE
            "aboveType": above_type,
            "aboveClientOrderId": above_client_order_id,
            "abovePrice": above_price,
            "aboveStopPrice": above_stop_price,
            "aboveTrailingDelta": above_trailing_delta,
            "aboveTimeInForce": above_time_in_force,
            "aboveIcebergQty": above_iceberg_qty,
            "aboveStrategyId": above_strategy_id,
            "aboveStrategyType": above_strategy_type,
            # BELOW
            "belowType": below_type,
            "belowClientOrderId": below_client_order_id,
            "belowPrice": below_price,
            "belowStopPrice": below_stop_price,
            "belowTrailingDelta": below_trailing_delta,
            "belowTimeInForce": below_time_in_force,
            "belowIcebergQty": below_iceberg_qty,
            "belowStrategyId": below_strategy_id,
            "belowStrategyType": below_strategy_type,
            # EXTRA
            "newOrderRespType": new_order_resp_type,
            "selfTradePreventionMode": self_trade_prevention_mode,
        }

        return await self._make_request("POST", url, True, params=params)

    async def oco_order_cancel(
        self,
        symbol: str,
        order_list_id: int | None = None,
        list_client_order_id: str | None = None,
        new_client_order_id: str | None = None,
    ) -> dict:
        """Отмена OCO ордера.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#cancel-oco-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/orderList"
        params = {
            "symbol": symbol,
            "orderListId": order_list_id,
            "listClientOrderId": list_client_order_id,
            "newClientOrderId": new_client_order_id,
        }

        return await self._make_request("DELETE", url, True, params=params)

    async def oco_order_get(
        self, order_list_id: int | None = None, orig_client_order_id: str | None = None
    ) -> dict:
        """Получение информации об OCO ордере.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-oco-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/orderList"
        params = {
            "orderListId": order_list_id,
            "origClientOrderId": orig_client_order_id,
        }

        return await self._make_request("GET", url, True, params=params)

    async def oco_orders_all(
        self,
        from_id: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение всех OCO ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-all-oco-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/allOrderList"
        params = {
            "fromId": from_id,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def oco_orders_open(self) -> list[dict]:
        """Получение активных OCO ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-open-oco-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/openOrderList"

        return await self._make_request("GET", url, True)

    # topic: account endpoints

    async def account(self) -> dict:
        """Получение информации об аккаунте (балансы, комиссии и т.д.).

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/account-endpoints#account-information-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/account"

        return await self._make_request("GET", url, True)

    async def order_get(
        self,
        symbol: str,
        order_id: int | None = None,
        orig_client_order_id: str | None = None,
    ) -> dict:
        """Получение информации об ордере.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-order-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/order"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "origClientOrderId": orig_client_order_id,
        }

        return await self._make_request("GET", url, True, params=params)

    async def all_orders(
        self,
        symbol: str,
        order_id: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение всех ордеров (активных, отмененных, исполненных) для символа.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#all-orders-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/allOrders"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def all_open_orders(
        self,
        symbol: str | None = None,
    ) -> list[dict]:
        """Получение всех ордеров активных ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/account-endpoints#current-open-orders-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/allOrders"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    async def my_trades(
        self,
        symbol: str,
        order_id: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        from_id: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение торговой истории аккаунта.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#account-trade-list-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/myTrades"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "startTime": start_time,
            "endTime": end_time,
            "fromId": from_id,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def order_count_usage(self) -> list[dict]:
        """Получение текущего использования лимитов ордеров.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-current-order-count-usage-trade
        """
        url = self._BASE_SPOT_URL + "/api/v3/rateLimit/order"

        return await self._make_request("GET", url, True)

    async def prevented_matches(
        self,
        symbol: str,
        prevented_match_id: int | None = None,
        order_id: int | None = None,
        from_prevented_match_id: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение предотвращенных совпадений.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-prevented-matches-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/myPreventedMatches"
        params = {
            "symbol": symbol,
            "preventedMatchId": prevented_match_id,
            "orderId": order_id,
            "fromPreventedMatchId": from_prevented_match_id,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def allocations(
        self,
        symbol: str,
        start_time: int | None = None,
        end_time: int | None = None,
        from_allocation_id: int | None = None,
        limit: int | None = None,
        order_id: int | None = None,
    ) -> list[dict]:
        """Получение распределений.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/spot-trading-endpoints#query-allocations-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/myAllocations"
        params = {
            "symbol": symbol,
            "startTime": start_time,
            "endTime": end_time,
            "fromAllocationId": from_allocation_id,
            "limit": limit,
            "orderId": order_id,
        }

        return await self._make_request("GET", url, True, params=params)

    async def commission_rates(self, symbol: str) -> dict:
        """Получение комиссионных ставок.

        https://developers.binance.com/docs/binance-spot-api-docs/rest-api/account-endpoints#query-commission-rates-user_data
        """
        url = self._BASE_SPOT_URL + "/api/v3/account/commission"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    # topic: futures market data

    async def futures_ping(self) -> dict:
        """Проверка подключения к REST API.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api#api-description
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/ping"

        return await self._make_request("GET", url)

    async def futures_server_time(self) -> dict:
        """Получение текущего времени сервера.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Check-Server-Time#api-description
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/time"

        return await self._make_request("GET", url)

    async def futures_exchange_info(self) -> dict:
        """Получение информации о символах рынка и текущих правилах биржевой торговли.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Exchange-Information#api-description
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/exchangeInfo"

        return await self._make_request("GET", url)

    async def futures_depth(self, symbol: str, limit: int | None = None) -> dict:
        """Получение книги ордеров.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Order-Book#request-parameters
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/depth"
        params = {"symbol": symbol, "limit": limit}

        return await self._make_request("GET", url, params=params)

    async def futures_trades(self, symbol: str, limit: int | None = None) -> list[dict]:
        """Получение последних сделок.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Recent-Trades-List
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/trades"
        params = {"symbol": symbol, "limit": limit}

        return await self._make_request("GET", url, params=params)

    async def futures_historical_trades(
        self, symbol: str, limit: int | None = None, from_id: int | None = None
    ) -> list[dict]:
        """Получение исторических сделок.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Old-Trades-Lookup
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/historicalTrades"
        params = {"symbol": symbol, "limit": limit, "fromId": from_id}

        return await self._make_request("GET", url, params=params)

    async def futures_agg_trades(
        self,
        symbol: str,
        from_id: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение агрегированных сделок на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Aggregate-Trades-List
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/aggTrades"
        params = {
            "symbol": symbol,
            "fromId": from_id,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_ticker_24hr(self, symbol: str | None = None) -> dict | list[dict]:
        """Получение статистики изменения цен и объема за 24 часа.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/24hr-Ticker-Price-Change-Statistics
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/ticker/24hr"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, params=params)

    async def futures_ticker_price(self, symbol: str | None = None) -> dict | list[dict]:
        """Получение последней цены тикера(ов).

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Symbol-Price-Ticker-v2
        """
        url = self._BASE_FUTURES_URL + "/fapi/v2/ticker/price"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, params=params)

    async def futures_klines(
        self,
        symbol: str,
        interval: str,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[list]:
        """Получение исторических свечей.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Kline-Candlestick-Data
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/klines"
        params = {
            "symbol": symbol,
            "interval": interval,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def open_interest(self, symbol: str) -> dict:
        """Получение открытого интереса тикера.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Open-Interest
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/openInterest"
        params = {"symbol": symbol}

        return await self._make_request(method="GET", url=url, params=params)

    async def futures_mark_price(self, symbol: str | None = None) -> dict | list[dict]:
        """Получение ставки финансирования и цены маркировки.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Mark-Price
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/premiumIndex"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, params=params)

    async def futures_funding_rate(
        self,
        symbol: str | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение истории ставок финансирования.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Get-Funding-Rate-History
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/fundingRate"
        params = {
            "symbol": symbol,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_funding_info(self) -> list[dict]:
        """Получение информации о ставках финансирования.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Funding-Rate-Info
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/fundingInfo"

        return await self._make_request("GET", url)

    async def open_interest_hist(
        self,
        symbol: str,
        period: str,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение истории открытого интереса.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Open-Interest-Statistics
        """
        url = self._BASE_FUTURES_URL + "/futures/data/openInterestHist"
        params = {
            "symbol": symbol,
            "period": period,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_long_short_ratio_accounts(
        self,
        symbol: str,
        period: str,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение соотношения лонг/шорт по аккаунтам.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Long-Short-Ratio
        """
        url = self._BASE_FUTURES_URL + "/futures/data/topLongShortAccountRatio"
        params = {
            "symbol": symbol,
            "period": period,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_long_short_ratio_positions(
        self,
        symbol: str,
        period: str,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение соотношения лонг/шорт по позициям.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Long-Short-Ratio
        """
        url = self._BASE_FUTURES_URL + "/futures/data/topLongShortPositionRatio"
        params = {
            "symbol": symbol,
            "period": period,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_global_long_short_ratio(
        self,
        symbol: str,
        period: str,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение глобального соотношения лонг/шорт.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Long-Short-Ratio
        """
        url = self._BASE_FUTURES_URL + "/futures/data/globalLongShortAccountRatio"
        params = {
            "symbol": symbol,
            "period": period,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_taker_long_short_ratio(
        self,
        symbol: str,
        period: str,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение соотношения лонг/шорт по тейкерам.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Taker-Buy-Sell-Volume
        """
        url = self._BASE_FUTURES_URL + "/futures/data/takerlongshortRatio"
        params = {
            "symbol": symbol,
            "period": period,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, params=params)

    async def futures_composite_index(self, symbol: str | None = None) -> list[dict]:
        """Получение композитного индекса.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Composite-Index-Symbol-Information
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/indexInfo"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, params=params)

    async def futures_api_trading_status(self) -> dict:
        """Получение статуса торгов API.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/market-data/rest-api/Query-Current-API-trading-status
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/apiTradingStatus"

        return await self._make_request("GET", url, True)

    # topic: futures account

    async def futures_account(self) -> dict:
        """Получение информации об аккаунте фьючерсов.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Account-Information-V3
        """
        url = self._BASE_FUTURES_URL + "/fapi/v3/account"
        return await self._make_request("GET", url, True)

    async def futures_balance(self) -> list[dict]:
        """Получение баланса фьючерсного аккаунта.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Futures-Account-Balance-V3
        """
        url = self._BASE_FUTURES_URL + "/fapi/v3/balance"

        return await self._make_request("GET", url, True)

    async def futures_multi_asset_mode(self, multi_assets_margin: bool | None = None) -> dict:
        """Изменение режима мультиактивной маржи.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Change-Multi-Assets-Mode
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/multiAssetsMargin"
        params = {"multiAssetsMargin": multi_assets_margin}

        return await self._make_request("POST", url, True, params=params)

    async def futures_multi_asset_mode_get(self) -> dict:
        """Получение режима мультиактивной маржи.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Get-Current-Multi-Assets-Mode
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/multiAssetsMargin"

        return await self._make_request("GET", url, True)

    # topic: futures trade

    async def futures_order_create(
        self,
        symbol: str,
        side: str,
        type: str,
        quantity: float | None = None,
        reduce_only: bool | None = None,
        price: float | None = None,
        new_client_order_id: str | None = None,
        stop_price: float | None = None,
        close_position: bool | None = None,
        activation_price: float | None = None,
        callback_rate: float | None = None,
        time_in_force: str | None = None,
        working_type: str | None = None,
        price_protect: bool | None = None,
        position_side: str | None = None,
        price_match: str | None = None,
        self_trade_prevention_mode: str | None = None,
        good_till_date: int | None = None,
    ) -> dict:
        """Создание нового ордера на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/New-Order
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/order"
        params = {
            "symbol": symbol,
            "side": side,
            "type": type,
            "quantity": quantity,
            "reduceOnly": reduce_only,
            "price": price,
            "newClientOrderId": new_client_order_id,
            "stopPrice": stop_price,
            "closePosition": close_position,
            "activationPrice": activation_price,
            "callbackRate": callback_rate,
            "timeInForce": time_in_force,
            "workingType": working_type,
            "priceProtect": price_protect,
            "positionSide": position_side,
            "priceMatch": price_match,
            "selfTradePreventionMode": self_trade_prevention_mode,
            "goodTillDate": good_till_date,
        }

        return await self._make_request("POST", url, True, params=params)

    async def futures_order_modify(
        self,
        order_id: int | None = None,
        orig_client_order_id: str | None = None,
        symbol: str | None = None,
        side: str | None = None,
        quantity: float | None = None,
        price: float | None = None,
        price_match: str | None = None,
    ) -> dict:
        """Изменение ордера на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Modify-Order
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/order"
        params = {
            "orderId": order_id,
            "origClientOrderId": orig_client_order_id,
            "symbol": symbol,
            "side": side,
            "quantity": quantity,
            "price": price,
            "priceMatch": price_match,
        }

        return await self._make_request("PUT", url, True, params=params)

    async def futures_order_get(
        self,
        symbol: str,
        order_id: int | None = None,
        orig_client_order_id: str | None = None,
    ) -> dict:
        """Получение информации об ордере на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Query-Order
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/order"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "origClientOrderId": orig_client_order_id,
        }

        return await self._make_request("GET", url, True, params=params)

    async def futures_orders_open(self, symbol: str | None = None) -> list[dict]:
        """Получение всех активных ордеров на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Current-All-Open-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/openOrders"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    async def futures_orders_all(
        self,
        symbol: str,
        order_id: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение всех ордеров на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/All-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/allOrders"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def futures_orders_cancel_all(self, symbol: str) -> dict:
        """Отмена всех активных ордеров на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Cancel-All-Open-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/allOpenOrders"
        params = {"symbol": symbol}

        return await self._make_request("DELETE", url, True, params=params)

    async def futures_countdown_cancel_all(
        self,
        symbol: str,
        countdown_time: int,
    ) -> dict:
        """Автоотмена всех активных ордеров через указанное время.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Auto-Cancel-All-Open-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/countdownCancelAll"
        params = {
            "symbol": symbol,
            "countdownTime": countdown_time,
        }

        return await self._make_request("POST", url, True, params=params)

    async def futures_position_info(self, symbol: str | None = None) -> list[dict]:
        """Получение информации о позициях на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Position-Information-V3
        """
        url = self._BASE_FUTURES_URL + "/fapi/v3/positionRisk"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    async def futures_my_trades(
        self,
        symbol: str,
        start_time: int | None = None,
        end_time: int | None = None,
        from_id: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение истории торгов на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Account-Trade-List
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/userTrades"
        params = {
            "symbol": symbol,
            "startTime": start_time,
            "endTime": end_time,
            "fromId": from_id,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def futures_income(
        self,
        symbol: str | None = None,
        income_type: str | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение истории доходов на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Get-Income-History
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/income"
        params = {
            "symbol": symbol,
            "incomeType": income_type,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def futures_leverage_change(self, symbol: str, leverage: int) -> dict:
        """Изменение кредитного плеча на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Change-Initial-Leverage
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/leverage"
        params = {"symbol": symbol, "leverage": leverage}

        return await self._make_request("POST", url, True, params=params)

    async def futures_margin_type_change(self, symbol: str, margin_type: str) -> dict:
        """Изменение типа маржи на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Change-Margin-Type
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/marginType"
        params = {"symbol": symbol, "marginType": margin_type}

        return await self._make_request("POST", url, True, params=params)

    async def futures_position_margin_modify(
        self,
        symbol: str,
        position_side: str,
        amount: float,
        type: int,
    ) -> dict:
        """Изменение изолированной маржи позиции.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Modify-Isolated-Position-Margin
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/positionMargin"
        params = {
            "symbol": symbol,
            "positionSide": position_side,
            "amount": amount,
            "type": type,
        }

        return await self._make_request("POST", url, True, params=params)

    async def futures_position_margin_history(
        self,
        symbol: str,
        type: int | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение истории изменений маржи позиции.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Get-Position-Margin-Change-History
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/positionMargin/history"
        params = {
            "symbol": symbol,
            "type": type,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def futures_commission_rate(self, symbol: str) -> dict:
        """Получение комиссионных ставок на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/User-Commission-Rate
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/commissionRate"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    async def futures_adl_quantile(self, symbol: str | None = None) -> list[dict]:
        """Получение информации об автоматической ликвидации.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Position-ADL-Quantile-Estimation
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/adlQuantile"
        params = {"symbol": symbol}

        return await self._make_request("GET", url, True, params=params)

    async def futures_force_orders(
        self,
        symbol: str | None = None,
        auto_close_type: str | None = None,
        start_time: int | None = None,
        end_time: int | None = None,
        limit: int | None = None,
    ) -> list[dict]:
        """Получение истории принудительных ордеров пользователя.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/User-s-Force-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/forceOrders"
        params = {
            "symbol": symbol,
            "autoCloseType": auto_close_type,
            "startTime": start_time,
            "endTime": end_time,
            "limit": limit,
        }

        return await self._make_request("GET", url, True, params=params)

    async def futures_api_key_permissions(self) -> dict:
        """Получение разрешений API ключа.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/account/rest-api/Get-API-Key-Permission
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/apiTradingStatus"

        return await self._make_request("GET", url, True)

    async def futures_order_cancel(
        self, symbol: str, order_id: int | None = None, orig_client_order_id: str | None = None
    ) -> dict:
        """Отмена активного ордера на фьючерсном рынке.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Cancel-Order
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/order"
        params = {
            "symbol": symbol,
            "orderId": order_id,
            "origClientOrderId": orig_client_order_id,
        }

        return await self._make_request("DELETE", url, params=params)

    async def futures_batch_orders_create(self, orders: list[dict]) -> list[dict]:
        """Создание множественных ордеров одновременно на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Place-Multiple-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/batchOrders"
        params = {"batchOrders": json.dumps(orders)}  # Нужен особый дамп

        return await self._make_request("POST", url, signed=True, params=params)

    async def futures_batch_orders_cancel(
        self,
        symbol: str,
        order_id_list: list[int] | None = None,
        orig_client_order_id_list: list[str] | None = None,
    ) -> list[dict]:
        """Отмена множественных ордеров на фьючерсах.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/trade/rest-api/Cancel-Multiple-Orders
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/batchOrders"
        params = {"symbol": symbol}

        if order_id_list:
            params["orderIdList"] = json.dumps(order_id_list)  # Нужен особый дамп

        if orig_client_order_id_list:
            params["origClientOrderIdList"] = json.dumps(  # Нужен особый дамп
                orig_client_order_id_list
            )

        return await self._make_request("DELETE", url, signed=True, params=params)

    # topic: user data streams

    async def listen_key(self) -> dict:
        """Создание ключа прослушивания для подключения к пользовательскому вебсокету.

        https://developers.binance.com/docs/binance-spot-api-docs/testnet/rest-api/user-data-stream-endpoints-deprecated#start-user-data-stream-user_stream-deprecated
        """
        warnings.warn(
            "These requests have been deprecated, which means we will remove them in the future. Please subscribe to the User Data Stream through the WebSocket API instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        url = self._BASE_SPOT_URL + "/api/v3/userDataStream"

        return await super()._make_request("POST", url, headers=self._get_headers("POST"))

    async def renew_listen_key(self, listen_key: str) -> dict:
        """Обновление ключа прослушивания для подключения к пользовательскому вебсокету.

        https://developers.binance.com/docs/binance-spot-api-docs/testnet/rest-api/user-data-stream-endpoints-deprecated#keepalive-user-data-stream-user_stream-deprecated
        """
        warnings.warn(
            "These requests have been deprecated, which means we will remove them in the future. Please subscribe to the User Data Stream through the WebSocket API instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        url = self._BASE_SPOT_URL + "/api/v3/userDataStream"
        params = {"listenKey": listen_key}

        return await super()._make_request(
            "PUT", url, params=params, headers=self._get_headers("PUT")
        )

    async def close_listen_key(self, listen_key: str) -> dict:
        """Закрытие ключа прослушивания для подключения к пользовательскому вебсокету.

        https://developers.binance.com/docs/binance-spot-api-docs/testnet/rest-api/user-data-stream-endpoints-deprecated#close-user-data-stream-user_stream-deprecated
        """
        warnings.warn(
            "[!IMPORTANT] These requests have been deprecated, which means we will remove them in the future. Please subscribe to the User Data Stream through the WebSocket API instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        url = self._BASE_SPOT_URL + "/api/v3/userDataStream"
        params = {"listenKey": listen_key}

        return await super()._make_request(
            "DELETE", url, params=params, headers=self._get_headers("DELETE")
        )

    # topic: futures user data streams

    async def futures_listen_key(self) -> dict:
        """Создание ключа прослушивания для подключения к пользовательскому вебсокету.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/user-data-streams/Start-User-Data-Stream#api-description
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/listenKey"

        return await super()._make_request("POST", url, headers=self._get_headers("POST"))

    async def futures_renew_listen_key(self) -> dict:
        """Обновление ключа прослушивания для подключения к пользовательскому вебсокету.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/user-data-streams
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/listenKey"

        return await super()._make_request("PUT", url, headers=self._get_headers("PUT"))

    async def futures_close_listen_key(self) -> dict:
        """Закрытие ключа прослушивания для подключения к пользовательскому вебсокету.

        https://developers.binance.com/docs/derivatives/usds-margined-futures/user-data-streams
        """
        url = self._BASE_FUTURES_URL + "/fapi/v1/listenKey"

        return await super()._make_request("DELETE", url, headers=self._get_headers("DELETE"))



__all__ = ["IUniWebsocketManager"]

from abc import ABC, abstractmethod
from collections.abc import Awaitable, Callable, Sequence
from typing import Any, overload

from loguru import logger as _logger

from unicex._base import BaseClient, Websocket
from unicex.enums import Timeframe
from unicex.types import LoggerLike

from .uni_client import IUniClient

type CallbackType = Callable[[Any], Awaitable[None]]


class IUniWebsocketManager(ABC):
    """Интерфейс менеджера асинхронных унифицированных вебсокетов."""

    def __init__(
        self, client: BaseClient | IUniClient | None = None, logger: LoggerLike | None = None
    ) -> None:
        """Инициализирует унифицированный менеджер вебсокетов.

        Параметры:
            client (`BaseClient | IUniClient | None`): Клиент или унифицированный клиент. Нужен для подключения к приватным топикам.
            logger (`LoggerLike | None`): Логгер для записи логов.
        """
        if isinstance(client, IUniClient):
            client = client.client
        self._client = client
        self._logger = logger or _logger

    def _make_wrapper(
        self, adapter_func: Callable[[dict], Any], callback: CallbackType
    ) -> CallbackType:
        """Создает обертку над callback, применяя адаптер к сырым сообщениям."""

        async def _wrapper(raw_msg: dict) -> None:
            try:
                adapted = adapter_func(raw_msg)
            except Exception as e:
                self._logger.warning(f"Failed to adapt message: {e}")
                return
            if isinstance(adapted, list):
                for item in adapted:
                    await callback(item)
            else:
                await callback(adapted)

        return _wrapper

    @overload
    def klines(
        self,
        callback: CallbackType,
        timeframe: Timeframe,
        *,
        symbol: str,
        symbols: None = None,
    ) -> Websocket: ...

    @overload
    def klines(
        self,
        callback: CallbackType,
        timeframe: Timeframe,
        *,
        symbol: None = None,
        symbols: Sequence[str],
    ) -> Websocket: ...

    @abstractmethod
    def klines(
        self,
        callback: CallbackType,
        timeframe: Timeframe,
        symbol: str | None = None,
        symbols: Sequence[str] | None = None,
    ) -> Websocket:
        """Открывает стрим свечей (spot) с унификацией сообщений.

        Параметры:
            callback (`CallbackType`): Асинхронная функция обратного вызова для обработки сообщений.
            timeframe (`Timeframe`): Временной интервал свечей.
            symbol (`str | None`): Один символ для подписки.
            symbols (`Sequence[str] | None`): Список символов для мультиплекс‑подключения.

        Должен быть указан либо `symbol`, либо `symbols`.

        Возвращает:
            `Websocket`: Экземпляр вебсокета для управления соединением.
        """
        ...

    @overload
    def futures_klines(
        self,
        callback: CallbackType,
        timeframe: Timeframe,
        *,
        symbol: str,
        symbols: None = None,
    ) -> Websocket: ...

    @overload
    def futures_klines(
        self,
        callback: CallbackType,
        timeframe: Timeframe,
        *,
        symbol: None = None,
        symbols: Sequence[str],
    ) -> Websocket: ...

    @abstractmethod
    def futures_klines(
        self,
        callback: CallbackType,
        timeframe: Timeframe,
        symbol: str | None = None,
        symbols: Sequence[str] | None = None,
    ) -> Websocket:
        """Открывает стрим свечей (futures) с унификацией сообщений.

        Параметры:
            callback (`CallbackType`): Асинхронная функция обратного вызова для обработки сообщений.
            timeframe (`Timeframe`): Временной интервал свечей.
            symbol (`str | None`): Один символ для подписки.
            symbols (`Sequence[str] | None`): Список символов для мультиплекс‑подключения.

        Должен быть указан либо `symbol`, либо `symbols`.

        Возвращает:
            `Websocket`: Экземпляр вебсокета.
        """
        ...

    @overload
    def trades(
        self,
        callback: CallbackType,
        *,
        symbol: str,
        symbols: None = None,
    ) -> Websocket: ...

    @overload
    def trades(
        self,
        callback: CallbackType,
        *,
        symbol: None = None,
        symbols: Sequence[str],
    ) -> Websocket: ...

    @abstractmethod
    def trades(
        self,
        callback: CallbackType,
        symbol: str | None = None,
        symbols: Sequence[str] | None = None,
    ) -> Websocket:
        """Открывает стрим сделок (spot) с унификацией сообщений.

        Параметры:
            callback (`CallbackType`): Асинхронная функция обратного вызова для обработки сообщений.
            symbol (`str | None`): Один символ для подписки.
            symbols (`Sequence[str] | None`): Список символов для мультиплекс‑подключения.

        Должен быть указан либо `symbol`, либо `symbols`.

        Возвращает:
            `Websocket`: Экземпляр вебсокета.
        """
        ...

    @overload
    def aggtrades(
        self,
        callback: CallbackType,
        *,
        symbol: str,
        symbols: None = None,
    ) -> Websocket: ...

    @overload
    def aggtrades(
        self,
        callback: CallbackType,
        *,
        symbol: None = None,
        symbols: Sequence[str],
    ) -> Websocket: ...

    @abstractmethod
    def aggtrades(
        self,
        callback: CallbackType,
        symbol: str | None = None,
        symbols: Sequence[str] | None = None,
    ) -> Websocket:
        """Открывает стрим агрегированных сделок (spot) с унификацией сообщений.

        Параметры:
            callback (`CallbackType`): Асинхронная функция обратного вызова для обработки сообщений.
            symbol (`str | None`): Один символ для подписки.
            symbols (`Sequence[str] | None`): Список символов для мультиплекс‑подключения.

        Должен быть указан либо `symbol`, либо `symbols`.

        Возвращает:
            `Websocket`: Экземпляр вебсокета.
        """
        ...

    @overload
    def futures_trades(
        self,
        callback: CallbackType,
        *,
        symbol: str,
        symbols: None = None,
    ) -> Websocket: ...

    @overload
    def futures_trades(
        self,
        callback: CallbackType,
        *,
        symbol: None = None,
        symbols: Sequence[str],
    ) -> Websocket: ...

    @abstractmethod
    def futures_trades(
        self,
        callback: CallbackType,
        symbol: str | None = None,
        symbols: Sequence[str] | None = None,
    ) -> Websocket:
        """Открывает стрим сделок (futures) с унификацией сообщений.

        Параметры:
            callback (`CallbackType`): Асинхронная функция обратного вызова для обработки сообщений.
            symbol (`str | None`): Один символ для подписки.
            symbols (`Sequence[str] | None`): Список символов для мультиплекс‑подключения.

        Должен быть указан либо `symbol`, либо `symbols`.

        Возвращает:
            `Websocket`: Экземпляр вебсокета.
        """
        ...

    @overload
    def futures_aggtrades(
        self,
        callback: CallbackType,
        *,
        symbol: str,
        symbols: None = None,
    ) -> Websocket: ...

    @overload
    def futures_aggtrades(
        self,
        callback: CallbackType,
        *,
        symbol: None = None,
        symbols: Sequence[str],
    ) -> Websocket: ...

    @abstractmethod
    def futures_aggtrades(
        self,
        callback: CallbackType,
        symbol: str | None = None,
        symbols: Sequence[str] | None = None,
    ) -> Websocket:
        """Открывает стрим агрегированных сделок (futures) с унификацией сообщений.

        Параметры:
            callback (`CallbackType`): Асинхронная функция обратного вызова для обработки сообщений.
            symbol (`str | None`): Один символ для подписки.
            symbols (`Sequence[str] | None`): Список символов для мультиплекс‑подключения.

        Должен быть указан либо `symbol`, либо `symbols`.

        Возвращает:
            `Websocket`: Экземпляр вебсокета.
        """
        ...



"""unicex - библиотека для работы с криптовалютными биржами, реализующая унифицированный интерфейс для работы с различными криптовалютными биржами."""

__all__ = [
    # Mappers
    "get_uni_client",
    "get_uni_websocket_manager",
    "get_exchange_info",
    # Exchanges info
    "load_exchanges_info",
    "start_exchanges_info",
    # Enums
    "MarketType",
    "Exchange",
    "Timeframe",
    "Side",
    # Types
    "TickerDailyDict",
    "TickerDailyItem",
    "KlineDict",
    "TradeDict",
    "AggTradeDict",
    "RequestMethod",
    "LoggerLike",
    "OpenInterestDict",
    "OpenInterestItem",
    "TickerInfoItem",
    "TickersInfoDict",
    "LiquidationDict",
    # Interfaces
    "IUniClient",
    "IUniWebsocketManager",
    # Base clients and websockets
    "Websocket",
    "BaseClient",
    # Binance
    "BinanceClient",
    "BinanceUniClient",
    "BinanceWebsocketManager",
    "BinanceUniWebsocketManager",
    "BinanceUserWebsocket",
    "BinanceExchangeInfo",
    # Bitget
    "BitgetClient",
    "BitgetUniClient",
    "BitgetUniWebsocketManager",
    "BitgetWebsocketManager",
    "BitgetUserWebsocket",
    "BitgetExchangeInfo",
    # Mexc
    "MexcClient",
    "MexcUniClient",
    "MexcUniWebsocketManager",
    "MexcWebsocketManager",
    "MexcUserWebsocket",
    "MexcExchangeInfo",
    # Bybit
    "BybitClient",
    "BybitUniClient",
    "BybitUniWebsocketManager",
    "BybitWebsocketManager",
    "BybitUserWebsocket",
    "BybitExchangeInfo",
    # Okx
    "OkxClient",
    "OkxUniClient",
    "OkxUniWebsocketManager",
    "OkxWebsocketManager",
    "OkxUserWebsocket",
    "OkxExchangeInfo",
    # Hyperliquid
    "HyperliquidClient",
    "HyperliquidUniClient",
    "HyperliquidUniWebsocketManager",
    "HyperliquidWebsocketManager",
    "HyperliquidUserWebsocket",
    "HyperliquidExchangeInfo",
    # Gateio
    "GateioClient",
    "GateioUniClient",
    "GateioUniWebsocketManager",
    "GateioWebsocketManager",
    "GateioUserWebsocket",
    "GateioExchangeInfo",
]

# ruff: noqa

# abstract & base
import asyncio
from ._abc import IUniClient, IUniWebsocketManager
from ._base import BaseClient, Websocket

# enums, mappers, types
from .enums import Exchange, MarketType, Side, Timeframe
from .mapper import get_uni_client, get_uni_websocket_manager, get_exchange_info
from .types import (
    TickerDailyDict,
    TickerDailyItem,
    KlineDict,
    TradeDict,
    AggTradeDict,
    RequestMethod,
    LoggerLike,
    OpenInterestDict,
    OpenInterestItem,
    TickerInfoItem,
    TickersInfoDict,
    LiquidationDict,
)

# exchanges

from .binance import (
    Client as BinanceClient,
    UniClient as BinanceUniClient,
    UniWebsocketManager as BinanceUniWebsocketManager,
    UserWebsocket as BinanceUserWebsocket,
    WebsocketManager as BinanceWebsocketManager,
    ExchangeInfo as BinanceExchangeInfo,
)

from .bitget import (
    Client as BitgetClient,
    UniClient as BitgetUniClient,
    UniWebsocketManager as BitgetUniWebsocketManager,
    UserWebsocket as BitgetUserWebsocket,
    WebsocketManager as BitgetWebsocketManager,
    ExchangeInfo as BitgetExchangeInfo,
)

from .bybit import (
    Client as BybitClient,
    UniClient as BybitUniClient,
    UniWebsocketManager as BybitUniWebsocketManager,
    UserWebsocket as BybitUserWebsocket,
    WebsocketManager as BybitWebsocketManager,
    ExchangeInfo as BybitExchangeInfo,
)

from .gate import (
    Client as GateioClient,
    UniClient as GateioUniClient,
    UniWebsocketManager as GateioUniWebsocketManager,
    UserWebsocket as GateioUserWebsocket,
    WebsocketManager as GateioWebsocketManager,
    ExchangeInfo as GateioExchangeInfo,
)

from .hyperliquid import (
    Client as HyperliquidClient,
    UniClient as HyperliquidUniClient,
    UniWebsocketManager as HyperliquidUniWebsocketManager,
    UserWebsocket as HyperliquidUserWebsocket,
    WebsocketManager as HyperliquidWebsocketManager,
    ExchangeInfo as HyperliquidExchangeInfo,
)

from .mexc import (
    Client as MexcClient,
    UniClient as MexcUniClient,
    UniWebsocketManager as MexcUniWebsocketManager,
    UserWebsocket as MexcUserWebsocket,
    WebsocketManager as MexcWebsocketManager,
    ExchangeInfo as MexcExchangeInfo,
)

from .okx import (
    Client as OkxClient,
    UniClient as OkxUniClient,
    UniWebsocketManager as OkxUniWebsocketManager,
    UserWebsocket as OkxUserWebsocket,
    WebsocketManager as OkxWebsocketManager,
    ExchangeInfo as OkxExchangeInfo,
)


async def load_exchanges_info() -> None:
    """Единожды загружает информацию о тикерах на всех биржах."""
    await asyncio.gather(
        BinanceExchangeInfo.load_exchange_info(),
        BitgetExchangeInfo.load_exchange_info(),
        BybitExchangeInfo.load_exchange_info(),
        GateioExchangeInfo.load_exchange_info(),
        HyperliquidExchangeInfo.load_exchange_info(),
        MexcExchangeInfo.load_exchange_info(),
        OkxExchangeInfo.load_exchange_info(),
    )


async def start_exchanges_info(parse_interval_seconds: int = 60 * 60) -> None:
    """Запускает цикл обновления информации о тикерах на всех биржах."""
    asyncio.gather(
        BinanceExchangeInfo.start(parse_interval_seconds),
        BitgetExchangeInfo.start(parse_interval_seconds),
        BybitExchangeInfo.start(parse_interval_seconds),
        GateioExchangeInfo.start(parse_interval_seconds),
        HyperliquidExchangeInfo.start(parse_interval_seconds),
        MexcExchangeInfo.start(parse_interval_seconds),
        OkxExchangeInfo.start(parse_interval_seconds),
    )



__all__ = ["ExchangeInfo"]

import aiohttp

from unicex._abc import IExchangeInfo
from unicex.types import TickerInfoItem

from .client import Client


class ExchangeInfo(IExchangeInfo):
    """Предзагружает информацию о тикерах для биржи Gateio."""

    exchange_name = "Gateio"
    """Название биржи, на которой работает класс."""

    @classmethod
    async def _load_spot_exchange_info(cls, session: aiohttp.ClientSession) -> None:
        """Загружает информацию о бирже для спотового рынка."""
        currency_pairs = await Client(session).currency_pairs()
        tickers_info: dict[str, TickerInfoItem] = {}
        for symbol_info in currency_pairs:
            try:
                tickers_info[symbol_info.get("id")] = TickerInfoItem(
                    tick_precision=int(symbol_info["precision"]),
                    tick_step=None,
                    size_precision=int(symbol_info["amount_precision"]),
                    size_step=None,
                    contract_size=1,
                )
            except ValueError as e:
                cls._logger.trace(
                    f"ValueError on {cls.exchange_name} by {symbol_info['symbol']}: {e}"
                )

        cls._tickers_info = tickers_info

    @classmethod
    async def _load_futures_exchange_info(cls, session: aiohttp.ClientSession) -> None:
        """Загружает информацию о бирже для фьючерсного рынка."""
        contracts = await Client(session).futures_contracts("usdt")
        tickers_info: dict[str, TickerInfoItem] = {}
        for contract in contracts:
            try:
                tickers_info[contract.get("name")] = TickerInfoItem(
                    tick_precision=None,
                    tick_step=float(contract["order_price_round"]),
                    size_precision=None,
                    size_step=float(contract["quanto_multiplier"]),
                    contract_size=float(contract["quanto_multiplier"]),
                )
            except ValueError as e:
                cls._logger.trace(f"ValueError on {cls.exchange_name} by {contract['name']}: {e}")

        cls._futures_tickers_info = tickers_info



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: PublicFuture.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 6, 31, 1, "", "PublicFuture.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12PublicFuture.proto"\xd3\x0b\n\x0f\x43ontractWrapper\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0f\n\x07\x63hannel\x18\x02 \x01(\t\x12\n\n\x02ts\x18\x03 \x01(\x03\x12$\n\x0borderChange\x18\xe9\x07 \x01(\x0b\x32\x0c.OrderChangeH\x00\x12$\n\x0b\x61ssetChange\x18\xea\x07 \x01(\x0b\x32\x0c.AssetChangeH\x00\x12$\n\x0b\x62onusNotify\x18\xeb\x07 \x01(\x0b\x32\x0c.BonusNotifyH\x00\x12&\n\x0cleverageMode\x18\xec\x07 \x01(\x0b\x32\r.LeverageModeH\x00\x12(\n\rnotifyGeneric\x18\xed\x07 \x01(\x0b\x32\x0e.NotifyGenericH\x00\x12\x34\n\x13liquidateRiskChange\x18\xee\x07 \x01(\x0b\x32\x14.LiquidateRiskChangeH\x00\x12,\n\x0fmarketOrderDeal\x18\xef\x07 \x01(\x0b\x32\x10.MarketOrderDealH\x00\x12,\n\x0forderDealDetail\x18\xf0\x07 \x01(\x0b\x32\x10.OrderDealDetailH\x00\x12\x30\n\x11\x63ontractPlanOrder\x18\xf1\x07 \x01(\x0b\x32\x12.ContractPlanOrderH\x00\x12\x32\n\x12positionModeResult\x18\xf2\x07 \x01(\x0b\x32\x13.PositionModeResultH\x00\x12*\n\x0epositionChange\x18\xf3\x07 \x01(\x0b\x32\x0f.PositionChangeH\x00\x12\x38\n\x15positionReverseChange\x18\xf4\x07 \x01(\x0b\x32\x16.PositionReverseChangeH\x00\x12*\n\x0euserPreference\x18\xf5\x07 \x01(\x0b\x32\x0f.UserPreferenceH\x00\x12\x30\n\x11\x63ontractStopOrder\x18\xf6\x07 \x01(\x0b\x32\x12.ContractStopOrderH\x00\x12\x38\n\x15\x63ontractUserRiskLimit\x18\xf7\x07 \x01(\x0b\x32\x16.ContractUserRiskLimitH\x00\x12\x38\n\x15\x63ontractStopPlanOrder\x18\xf8\x07 \x01(\x0b\x32\x16.ContractStopPlanOrderH\x00\x12\x32\n\x12\x63ontractTrackOrder\x18\xf9\x07 \x01(\x0b\x32\x13.ContractTrackOrderH\x00\x12"\n\norderChase\x18\xfa\x07 \x01(\x0b\x32\x0b.OrderChaseH\x00\x12"\n\nindexPrice\x18\xd1\x0f \x01(\x0b\x32\x0b.IndexPriceH\x00\x12&\n\x0c\x63ommonConfig\x18\xd2\x0f \x01(\x0b\x32\r.CommonConfigH\x00\x12(\n\rcontractExtra\x18\xd3\x0f \x01(\x0b\x32\x0e.ContractExtraH\x00\x12\x1e\n\x08\x63ontract\x18\xd4\x0f \x01(\x0b\x32\t.ContractH\x00\x12\x16\n\x04\x64\x65\x61l\x18\xd5\x0f \x01(\x0b\x32\x05.DealH\x00\x12\x18\n\x05\x64\x65\x61ls\x18\xd6\x0f \x01(\x0b\x32\x06.DealsH\x00\x12\x32\n\x12marketOrderSetting\x18\xb4\x10 \x01(\x0b\x32\x13.MarketOrderSettingH\x00\x12\x1c\n\x07wsKline\x18\xb5\x10 \x01(\x0b\x32\x08.WsKLineH\x00\x12$\n\x0b\x66undingRate\x18\xb6\x10 \x01(\x0b\x32\x0c.FundingRateH\x00\x12 \n\tfairPrice\x18\xdd\x0f \x01(\x0b\x32\n.FairPriceH\x00\x12(\n\rdepthStepInfo\x18\xde\x0f \x01(\x0b\x32\x0e.DepthStepInfoH\x00\x12 \n\tdepthInfo\x18\xdf\x0f \x01(\x0b\x32\n.DepthInfoH\x00\x12"\n\ndepthMerge\x18\xe0\x0f \x01(\x0b\x32\x0b.DepthMergeH\x00\x12&\n\x0ctickerDetail\x18\xe1\x0f \x01(\x0b\x32\r.TickerDetailH\x00\x12"\n\ntickerList\x18\xe2\x0f \x01(\x0b\x32\x0b.TickerListH\x00\x12\x1e\n\x08tickList\x18\xe3\x0f \x01(\x0b\x32\t.TickListH\x00\x42\t\n\x07payload"M\n\tDepthInfo\x12\x14\n\x04\x62ids\x18\x01 \x03(\x0b\x32\x06.Order\x12\x14\n\x04\x61sks\x18\x02 \x03(\x0b\x32\x06.Order\x12\x14\n\x0clastUpdateId\x18\x03 \x01(\x03"8\n\nDepthMerge\x12\x14\n\x04\x62ids\x18\x01 \x03(\x0b\x32\x06.Order\x12\x14\n\x04\x61sks\x18\x02 \x03(\x0b\x32\x06.Order"(\n\x05Order\x12\r\n\x05price\x18\x01 \x01(\t\x12\x10\n\x08quantity\x18\x02 \x01(\t"\x99\x03\n\x0cTickerDetail\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x11\n\tlastPrice\x18\x02 \x01(\t\x12\x14\n\x0criseFallRate\x18\x03 \x01(\t\x12\x11\n\tfairPrice\x18\x04 \x01(\t\x12\x12\n\nindexPrice\x18\x05 \x01(\t\x12\x10\n\x08volume24\x18\x06 \x01(\t\x12\x10\n\x08\x61mount24\x18\x07 \x01(\t\x12\x13\n\x0bmaxBidPrice\x18\x08 \x01(\t\x12\x13\n\x0bminAskPrice\x18\t \x01(\t\x12\x14\n\x0clower24Price\x18\n \x01(\t\x12\x13\n\x0bhigh24Price\x18\x0b \x01(\t\x12\x11\n\ttimestamp\x18\x0c \x01(\x03\x12\x0c\n\x04\x62id1\x18\r \x01(\t\x12\x0c\n\x04\x61sk1\x18\x0e \x01(\t\x12\x0f\n\x07holdVol\x18\x0f \x01(\t\x12\x15\n\rriseFallValue\x18\x10 \x01(\t\x12\x13\n\x0b\x66undingRate\x18\x11 \x01(\t\x12\x0c\n\x04zone\x18\x12 \x01(\t\x12\x15\n\rriseFallRates\x18\x13 \x03(\t\x12\x1f\n\x17riseFallRatesOfTimezone\x18\x14 \x03(\t",\n\nTickerList\x12\x1e\n\x07tickers\x18\x01 \x03(\x0b\x32\r.TickerDetail"D\n\x0b\x46undingRate\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04rate\x18\x02 \x01(\t\x12\x17\n\x0fnextFundingTime\x18\x03 \x01(\x03"+\n\nIndexPrice\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t"*\n\tFairPrice\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t"O\n\x0bOrderChange\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\t"3\n\x0b\x41ssetChange\x12\r\n\x05\x61sset\x18\x01 \x01(\t\x12\x15\n\rbalanceChange\x18\x02 \x01(\t"S\n\x0fMarketOrderDeal\x12\x0e\n\x06\x64\x65\x61lId\x18\x01 \x01(\t\x12\x0f\n\x07orderId\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\t"F\n\x0ePositionChange\x12\x12\n\npositionId\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\x12\x10\n\x08quantity\x18\x03 \x01(\t"\x1d\n\x05\x44\x65\x61ls\x12\x14\n\x05\x64\x65\x61ls\x18\x01 \x03(\x0b\x32\x05.Deal"I\n\x04\x44\x65\x61l\x12\t\n\x01p\x18\x01 \x01(\t\x12\t\n\x01v\x18\x02 \x01(\t\x12\t\n\x01t\x18\x03 \x01(\x05\x12\t\n\x01o\x18\x04 \x01(\x05\x12\t\n\x01m\x18\x05 \x01(\x05\x12\n\n\x02ts\x18\x06 \x01(\x03"L\n\x0b\x42onusNotify\x12\x0f\n\x07\x62onusId\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0e\n\x06\x61mount\x18\x03 \x01(\t\x12\x0e\n\x06status\x18\x04 \x01(\t",\n\x0cLeverageMode\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04mode\x18\x02 \x01(\t"A\n\rNotifyGeneric\x12\x11\n\tmessageId\x18\x01 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t"L\n\x13LiquidateRiskChange\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x11\n\triskLevel\x18\x02 \x01(\t\x12\x12\n\nmarginCall\x18\x03 \x01(\t"T\n\x0fOrderDealDetail\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x0f\n\x07tradeId\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\t"4\n\x11\x43ontractPlanOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t""\n\x12PositionModeResult\x12\x0c\n\x04mode\x18\x01 \x01(\t"A\n\x15PositionReverseChange\x12\x12\n\npositionId\x18\x01 \x01(\t\x12\x14\n\x0c\x63hangeAmount\x18\x02 \x01(\t"2\n\x0eUserPreference\x12\x0e\n\x06userId\x18\x01 \x01(\t\x12\x10\n\x08settings\x18\x02 \x01(\t"7\n\x11\x43ontractStopOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x11\n\tstopPrice\x18\x02 \x01(\t":\n\x15\x43ontractUserRiskLimit\x12\x0e\n\x06userId\x18\x01 \x01(\t\x12\x11\n\triskLimit\x18\x02 \x01(\t">\n\x15\x43ontractStopPlanOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x14\n\x0ctriggerPrice\x18\x02 \x01(\t"9\n\x12\x43ontractTrackOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x12\n\ntrackPrice\x18\x02 \x01(\t"1\n\nOrderChase\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x12\n\nchasePrice\x18\x02 \x01(\t"*\n\x0c\x43ommonConfig\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"5\n\rContractExtra\x12\x12\n\ncontractId\x18\x01 \x01(\t\x12\x10\n\x08leverage\x18\x02 \x01(\t".\n\x08\x43ontract\x12\x12\n\ncontractId\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t"6\n\x12MarketOrderSetting\x12\x11\n\tsettingId\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"a\n\x07WsKLine\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04open\x18\x02 \x01(\t\x12\x0c\n\x04high\x18\x03 \x01(\t\x12\x0b\n\x03low\x18\x04 \x01(\t\x12\r\n\x05\x63lose\x18\x05 \x01(\t\x12\x0e\n\x06volume\x18\x06 \x01(\t",\n\rDepthStepInfo\x12\x0c\n\x04step\x18\x01 \x01(\t\x12\r\n\x05\x63ount\x18\x02 \x01(\t"(\n\x08TickList\x12\x1c\n\x05ticks\x18\x01 \x03(\x0b\x32\r.TickerDetailb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "PublicFuture_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_CONTRACTWRAPPER"]._serialized_start = 23
    _globals["_CONTRACTWRAPPER"]._serialized_end = 1514
    _globals["_DEPTHINFO"]._serialized_start = 1516
    _globals["_DEPTHINFO"]._serialized_end = 1593
    _globals["_DEPTHMERGE"]._serialized_start = 1595
    _globals["_DEPTHMERGE"]._serialized_end = 1651
    _globals["_ORDER"]._serialized_start = 1653
    _globals["_ORDER"]._serialized_end = 1693
    _globals["_TICKERDETAIL"]._serialized_start = 1696
    _globals["_TICKERDETAIL"]._serialized_end = 2105
    _globals["_TICKERLIST"]._serialized_start = 2107
    _globals["_TICKERLIST"]._serialized_end = 2151
    _globals["_FUNDINGRATE"]._serialized_start = 2153
    _globals["_FUNDINGRATE"]._serialized_end = 2221
    _globals["_INDEXPRICE"]._serialized_start = 2223
    _globals["_INDEXPRICE"]._serialized_end = 2266
    _globals["_FAIRPRICE"]._serialized_start = 2268
    _globals["_FAIRPRICE"]._serialized_end = 2310
    _globals["_ORDERCHANGE"]._serialized_start = 2312
    _globals["_ORDERCHANGE"]._serialized_end = 2391
    _globals["_ASSETCHANGE"]._serialized_start = 2393
    _globals["_ASSETCHANGE"]._serialized_end = 2444
    _globals["_MARKETORDERDEAL"]._serialized_start = 2446
    _globals["_MARKETORDERDEAL"]._serialized_end = 2529
    _globals["_POSITIONCHANGE"]._serialized_start = 2531
    _globals["_POSITIONCHANGE"]._serialized_end = 2601
    _globals["_DEALS"]._serialized_start = 2603
    _globals["_DEALS"]._serialized_end = 2632
    _globals["_DEAL"]._serialized_start = 2634
    _globals["_DEAL"]._serialized_end = 2707
    _globals["_BONUSNOTIFY"]._serialized_start = 2709
    _globals["_BONUSNOTIFY"]._serialized_end = 2785
    _globals["_LEVERAGEMODE"]._serialized_start = 2787
    _globals["_LEVERAGEMODE"]._serialized_end = 2831
    _globals["_NOTIFYGENERIC"]._serialized_start = 2833
    _globals["_NOTIFYGENERIC"]._serialized_end = 2898
    _globals["_LIQUIDATERISKCHANGE"]._serialized_start = 2900
    _globals["_LIQUIDATERISKCHANGE"]._serialized_end = 2976
    _globals["_ORDERDEALDETAIL"]._serialized_start = 2978
    _globals["_ORDERDEALDETAIL"]._serialized_end = 3062
    _globals["_CONTRACTPLANORDER"]._serialized_start = 3064
    _globals["_CONTRACTPLANORDER"]._serialized_end = 3116
    _globals["_POSITIONMODERESULT"]._serialized_start = 3118
    _globals["_POSITIONMODERESULT"]._serialized_end = 3152
    _globals["_POSITIONREVERSECHANGE"]._serialized_start = 3154
    _globals["_POSITIONREVERSECHANGE"]._serialized_end = 3219
    _globals["_USERPREFERENCE"]._serialized_start = 3221
    _globals["_USERPREFERENCE"]._serialized_end = 3271
    _globals["_CONTRACTSTOPORDER"]._serialized_start = 3273
    _globals["_CONTRACTSTOPORDER"]._serialized_end = 3328
    _globals["_CONTRACTUSERRISKLIMIT"]._serialized_start = 3330
    _globals["_CONTRACTUSERRISKLIMIT"]._serialized_end = 3388
    _globals["_CONTRACTSTOPPLANORDER"]._serialized_start = 3390
    _globals["_CONTRACTSTOPPLANORDER"]._serialized_end = 3452
    _globals["_CONTRACTTRACKORDER"]._serialized_start = 3454
    _globals["_CONTRACTTRACKORDER"]._serialized_end = 3511
    _globals["_ORDERCHASE"]._serialized_start = 3513
    _globals["_ORDERCHASE"]._serialized_end = 3562
    _globals["_COMMONCONFIG"]._serialized_start = 3564
    _globals["_COMMONCONFIG"]._serialized_end = 3606
    _globals["_CONTRACTEXTRA"]._serialized_start = 3608
    _globals["_CONTRACTEXTRA"]._serialized_end = 3661
    _globals["_CONTRACT"]._serialized_start = 3663
    _globals["_CONTRACT"]._serialized_end = 3709
    _globals["_MARKETORDERSETTING"]._serialized_start = 3711
    _globals["_MARKETORDERSETTING"]._serialized_end = 3765
    _globals["_WSKLINE"]._serialized_start = 3767
    _globals["_WSKLINE"]._serialized_end = 3864
    _globals["_DEPTHSTEPINFO"]._serialized_start = 3866
    _globals["_DEPTHSTEPINFO"]._serialized_end = 3910
    _globals["_TICKLIST"]._serialized_start = 3912
    _globals["_TICKLIST"]._serialized_end = 3952
# @@protoc_insertion_point(module_scope)
